@@ -1,3 +1,4 @@
+"""Module containing the logic for ``twine check``."""
 # Copyright 2018 Dustin Ingram
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,16 +15,19 @@
 import argparse
 import cgi
 import io
+import logging
 import re
-import sys
-import textwrap
-from typing import IO, List, Optional, Tuple, cast
+from typing import List, Optional, Tuple, cast
 
 import readme_renderer.rst
+from rich import print
 
 from twine import commands
 from twine import package as package_file
 
+logger = logging.getLogger(__name__)
+
+
 _RENDERERS = {
     None: readme_renderer.rst,  # Default if description_content_type is None
     "text/plain": None,  # Rendering cannot fail
@@ -44,27 +48,19 @@ _REPORT_RE = re.compile(
 )
 
 
-class _WarningStream:
-    def __init__(self) -> None:
-        self.output = io.StringIO()
-
-    def write(self, text: str) -> None:
+class _WarningStream(io.StringIO):
+    def write(self, text: str) -> int:
         matched = _REPORT_RE.search(text)
+        if matched:
+            line = matched.group("line")
+            level_text = matched.group("level").capitalize()
+            message = matched.group("message").rstrip("\r\n")
+            text = f"line {line}: {level_text}: {message}\n"
 
-        if not matched:
-            self.output.write(text)
-            return
-
-        self.output.write(
-            "line {line}: {level_text}: {message}\n".format(
-                level_text=matched.group("level").capitalize(),
-                line=matched.group("line"),
-                message=matched.group("message").rstrip("\r\n"),
-            )
-        )
+        return super().write(text)
 
     def __str__(self) -> str:
-        return self.output.getvalue()
+        return self.getvalue().strip()
 
 
 def _check_file(
@@ -89,7 +85,7 @@ def _check_file(
     content_type, params = cgi.parse_header(description_content_type)
     renderer = _RENDERERS.get(content_type, _RENDERERS[None])
 
-    if description in {None, "UNKNOWN\n\n\n"}:
+    if description is None or description.rstrip() == "UNKNOWN":
         warnings.append("`long_description` missing.")
     elif renderer:
         rendering_result = renderer.render(
@@ -103,49 +99,69 @@ def _check_file(
 
 def check(
     dists: List[str],
-    output_stream: IO[str] = sys.stdout,
     strict: bool = False,
 ) -> bool:
+    """Check that a distribution will render correctly on PyPI and display the results.
+
+    This is currently only validates ``long_description``, but more checks could be
+    added; see https://github.com/pypa/twine/projects/2.
+
+    :param dists:
+        The distribution files to check.
+    :param output_stream:
+        The destination of the resulting output.
+    :param strict:
+        If ``True``, treat warnings as errors.
+
+    :return:
+        ``True`` if there are rendering errors, otherwise ``False``.
+    """
     uploads = [i for i in commands._find_dists(dists) if not i.endswith(".asc")]
     if not uploads:  # Return early, if there are no files to check.
-        output_stream.write("No files to check.\n")
+        logger.error("No files to check.")
         return False
 
     failure = False
 
     for filename in uploads:
-        output_stream.write("Checking %s: " % filename)
+        print(f"Checking {filename}: ", end="")
         render_warning_stream = _WarningStream()
         warnings, is_ok = _check_file(filename, render_warning_stream)
 
         # Print the status and/or error
         if not is_ok:
             failure = True
-            output_stream.write("FAILED\n")
-
-            error_text = (
-                "`long_description` has syntax errors in markup and "
-                "would not be rendered on PyPI.\n"
+            print("[red]FAILED[/red]")
+            logger.error(
+                "`long_description` has syntax errors in markup"
+                " and would not be rendered on PyPI."
+                f"\n{render_warning_stream}"
             )
-            output_stream.write(textwrap.indent(error_text, "  "))
-            output_stream.write(textwrap.indent(str(render_warning_stream), "    "))
         elif warnings:
             if strict:
                 failure = True
-                output_stream.write("FAILED, due to warnings\n")
+                print("[red]FAILED due to warnings[/red]")
             else:
-                output_stream.write("PASSED, with warnings\n")
+                print("[yellow]PASSED with warnings[/yellow]")
         else:
-            output_stream.write("PASSED\n")
+            print("[green]PASSED[/green]")
 
         # Print warnings after the status and/or error
         for message in warnings:
-            output_stream.write("  warning: " + message + "\n")
+            logger.warning(message)
 
     return failure
 
 
 def main(args: List[str]) -> bool:
+    """Execute the ``check`` command.
+
+    :param args:
+        The command-line arguments.
+
+    :return:
+        The exit status of the ``check`` command.
+    """
     parser = argparse.ArgumentParser(prog="twine check")
     parser.add_argument(
         "dists",
